#version 120
// sample BPlot2D shader -- simple y(x) graph via indicator: y == y(x)

// shader parameters

uniform vec2 screen; // screen pixel sizes
uniform vec4 world;  // box rectangle: (x0, y0, x1-x0, y1-y0)


// graph
float f1(vec2 p)
{
  float x = sin(p.x+sqrt(abs(p.y))), y = cos(p.y + p.x*p.x);
  return 4.5*x*x*x - 0.5*x*x*y + 4.0*x*y*y - y*y*y + 3.5*x*y - 1.5*x*x + 3.0*y*y - 2.0*x + 1.5*y;
}

float f2(vec2 p)
{
  float x = sin(p.x+1.414213562373*p.y), y = cos(p.y*p.x - p.x*p.x);
  return 5.0*(x*x)*(x*x - 0.5*y) + y*y*y - sqrt(abs(x) + abs(y));
}

vec3 map2d(vec2 p)
{
  float a = f1(p), b = f2(p);
  float intensity = clamp(0.6366198 * atan(max(abs(a), abs(b))), 0.0, 1.0);
  return vec3
  (
    (a < 0.0 || b < 0.0? intensity: 0.0),
    (0.0 < b? intensity: 0.0),
    (0.0 < a? intensity: 0.0)
  );
}

// auxiliary functions

// maps pixel screen coordinates pc to "world" coordinates
vec2 screen_to_world(vec2 pc)
{
  return world.xy + pc / screen * world.zw;
}
		   
// supersampling
const int offsets = 13;
vec2 off[offsets] = vec2[]
(
  vec2(0.50, 0.50),
  vec2(0.40, 0.65),
  vec2(0.65, 0.60),
  vec2(0.60, 0.35),
  vec2(0.35, 0.40),
  vec2(0.15, 0.80),
  vec2(0.80, 0.85),
  vec2(0.85, 0.20),
  vec2(0.20, 0.15),
  vec2(0.10, 0.45),
  vec2(0.45, 0.90),
  vec2(0.90, 0.55),
  vec2(0.55, 0.10)
);
		   
void main(void)
{
  vec3 c = vec3(0, 0, 0);
  for (int i = 0; i < offsets; ++i)
    c += map2d(screen_to_world(gl_FragCoord.xy + off[i]));

  gl_FragColor = vec4(c/offsets, 1.0);
}
