﻿/*
* This file is part of AV Orchid.
* Copyright (c) 2016, Dmitri R. Kuvshinov <evetro.here@gmail.com>
*
* Permission to use, copy, modify, and/or distribute this software for
* any purpose with or without fee is hereby granted, provided that the
* above copyright notice and this permission notice appear in all copies.
*
* THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
* WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
* ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
* WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
* ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
* OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
/**
* @file Plotting2D.hpp
* @brief интерфейс рисования графика
*/
#pragma once
#include <avo/Vec2.hpp>
#include <avo/Color.hpp>

/// Вектор/точка с координатами x, y.
using Vec_2 = avo::Vec_2d;

/// 4-компонентный цвет.
using avo::Color;

/// Создать окно, подготовить внутренние структуры данных для начала рисования.
/// Повторный вызов только меняет параметры окна рисования: название, ширину, высоту (в пикселях).
/// Возвращает false если окно невозможно создать.
bool setup_plotting_window(
  const char *title = "Plotting",
  int window_width = 800,
  int window_height = 600);

/// Начать рисование нового графика.
/// Возвращает false если окно было закрыто и рисование невозможно.
bool start_plotting(
  Color vertex = avo::RED,   ///< цвет вершины графика
  float vertex_px = 3.f,     ///< диаметр вершины графика в пикселях, 0, чтобы не рисовать
  Color edge = avo::YELLOW,  ///< цвет линии графика (отрезка ломаной)
  float edge_px = 2.f        ///< толщина линии графика в пикселях, 0, чтобы не рисовать
  );

/// Получить область отображения всего графика.
/// Возвращает false если окно было закрыто и рисование невозможно.
bool plotting_view_all();

/// Установить координаты отображаемой области.
/// Возвращает false если окно было закрыто и рисование невозможно.
bool plotting_view(const Vec_2 &pmin, const Vec_2 &pmax);

/// Установить координаты отображаемой области (координаты).
/// Возвращает false если окно было закрыто и рисование невозможно.
inline bool plotting_view(
  double x_min, double y_min, ///< нижний левый угол
  double x_max, double y_max  ///< верхний правый угол
  )
{
  return plotting_view(Vec_2(x_min, y_min), Vec_2(x_max, y_max));
}

/// Передать следующую точку для размещения в графике.
/// Возвращает false если окно было закрыто и рисование невозможно.
bool plot(const Vec_2 &pos);

/// Передать следующую точку для размещения в графике (координаты).
/// Возвращает false если окно было закрыто и рисование невозможно.
inline bool plot(double x, double y)
{
  return plot(Vec_2(x, y));
}

/// Удалить все графики.
/// Возвращает false если окно было закрыто и рисование невозможно.
bool clear_plotting();

/// Проверить, закрыто ли окно.
bool is_already_closed();