#version 110

// This GLSL fragment shader implements "Plasma" effect,
// see AV Orchid/GLSLPlasma to find out details.
// Author: Dmitri Kuvshinov.

// shader parameters

uniform sampler1D palette;
uniform float t;
uniform float pixel_units_coef;
uniform float width, height;

// constants

const float pi2 = 6.283185307179586;
const vec4 st_arg_coefs = pi2/8192.0 * vec4(1.1,  1.5,  0.375, 0.7);
const vec4 st_arg_offs  = pi2/512.0  * vec4(17.0, 107.0, 210., 444.0);
const vec4 ones = vec4(1.0, 1.0, 1.0, 1.0);

const vec4 x_coefs    = vec4(3.0/7.0, 0.75,  12.0/7.0, 3.0/5.0);
const vec4 x_st_coefs = vec4(3.0, 1.0, -2.0, -1.0);

const vec4 y_coefs    = vec4(4.0/3.0, 28.0/3.0, 0.5, 0.0);
const vec4 y_st_coefs = vec4(-2.0,   -7.0, -1.0, 0.5);

// shader procedure

void main(void)
{
  vec2 coord = pixel_units_coef * gl_FragCoord.xy;
  vec4 st = (width + height) * (sin(t * st_arg_coefs + st_arg_offs) + ones);
  vec4 x  = coord.xxxx - (width  * x_coefs + st * x_st_coefs);
  vec4 y  = coord.yyyy - (height * y_coefs + st.zwyx * y_st_coefs);
  vec4 ss = sin(sqrt(x * x + y * y)) + ones;
  float plasma = 0.125 * (ss.x + ss.y + ss.z + ss.w);

  gl_FragColor = texture1D(palette, plasma);
}
