﻿/*
 * This file is part of AV Orchid.
 * Copyright (c) 2014, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/*
   Данный модуль (С90) предоставляет точку входа для "нативных" приложений Win32,
   из которой вызывается стандартная точка входа main(argc, argv).
   При сочетании нежелания наблюдать окно консоли Windows и менять свой код под Windows
   добавьте данный модуль в проект.
   Использует средства, предоставляемые самим Win32 API.
   
   UCS2 перекодируется в UTF-8.
   В случае невозможности выделить память выходит с возвращением стандартного кода ошибки.
   Остальные ошибки игнорируются. Параметры, которые не удалось перекодировать, отбрасываются.
   Гарантируется, что количество параметров при вызове main будет не меньше одного,
   однако даже первый параметр может указывать на пустую строку (в случае возникновения ошибок).

   См. http://msdn.microsoft.com/en-us/library/windows/desktop/bb776391%28v=vs.85%29.aspx
       http://msdn.microsoft.com/en-us/library/windows/desktop/dd374130%28v=vs.85%29.aspx
*/
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <shellapi.h>
#include <stdlib.h>

static char* argv_on_error[2];
static char empty_arg;


/* параметры самой функции игнорируются */
int WINAPI WinMain
  (
    _In_ HINSTANCE hInstance,
    _In_opt_ HINSTANCE hPrevInstance,
    _In_ LPSTR lpCmdLine,
    _In_ int nShowCmd
  )    
{
  int main(int argc, char **argv);
  LPWSTR *argw;
  char **argv;
  int argc;

  argv = NULL;
  /* разобрать строку на отдельные параметры средствами Win32 */
  argw = CommandLineToArgvW(GetCommandLineW(), &argc);
  
  /* перекодирование UCS2 -> UTF-8 */
  if (argw && argc > 0)
  {
    int i, buf_sz, j;
    /* выделить место под массив указателей на перекодированные параметры */
    buf_sz = (argc + 1) * sizeof(char*);
    argv = malloc(buf_sz);
    if (!argv)
      return EXIT_FAILURE;
    ZeroMemory(argv, buf_sz);

    for (j = i = 0; i < argc; ++i)
    {
      /* узнать размер буфера */
      buf_sz = WideCharToMultiByte(CP_UTF8, 0, argw[i], -1, NULL, 0, NULL, NULL);
      if (buf_sz > 0)
      {
        /* выделить буфер для параметра i */
        argv[j] = malloc(buf_sz);
        if (!argv[j])
          return EXIT_FAILURE;

        /* перекодировать */
        if (WideCharToMultiByte(CP_UTF8, 0, argw[i], -1, argv[j], buf_sz, NULL, NULL) == 0)
          free(argv[j]);
        else
          ++j; /* успешно */
      }
    }

    argc = j;
  }

  /* общая ошибка */
  if (!argv || argc < 1)
  {
    argv_on_error[0] = argv_on_error[1] = &empty_arg;
    argv = argv_on_error;
    argc = 1;
  }

  /* вызов стандартной точки входа */
  return main(argc, argv);
}
